Package: MachineShop
Type: Package
Title: Machine Learning Models and Tools
Version: 3.9.4
Date: 2026-09-04
Authors@R: person(c("Brian", "J"), "Smith",
                  email = "brian-j-smith@uiowa.edu",
                  role = c("aut", "cre"))
Author: Brian J Smith [aut, cre]
Maintainer: Brian J Smith <brian-j-smith@uiowa.edu>
Description: Meta-package for statistical and machine learning with a unified
    interface for model fitting, prediction, performance assessment, and
    presentation of results.  Approaches for model fitting and prediction of
    numerical, categorical, or censored time-to-event outcomes include
    traditional regression models, regularization methods, tree-based methods,
    support vector machines, neural networks, ensembles, data preprocessing,
    filtering, and model tuning and selection.  Performance metrics are provided
    for model assessment and can be estimated with independent test sets, split
    sampling, cross-validation, or bootstrap resampling.  Resample estimation
    can be executed in parallel for faster processing and nested in cases of
    model tuning and selection.  Modeling results can be summarized with
    descriptive statistics; calibration curves; variable importance; partial
    dependence plots; confusion matrices; and ROC, lift, and other performance
    curves.
Depends: R (>= 4.1.0)
Imports: abind, cli (>= 3.1.0), dials (>= 0.0.4), foreach, ggplot2 (>=
        3.4.0), kernlab, magrittr, Matrix (>= 1.5-0), methods, nnet,
        party, polspline, progress, recipes (>= 1.0.0), rlang, rsample
        (>= 1.1.0), Rsolnp, survival, tibble, utils
Suggests: adabag, BART, bartMachine, C50, censored, cluster,
        doParallel, e1071, earth, elasticnet, generics, gbm, glmnet,
        gridExtra, Hmisc, kableExtra, kknn, knitr, lars, MASS, mboost,
        mda, ParBayesianOptimization, parsnip (>= 1.1.0), partykit,
        pls, pso, randomForest, randomForestSRC, ranger,
        rBayesianOptimization, rmarkdown, rms, rpart, testthat, tree,
        xgboost (>= 3.1.2)
Additional_repositories: https://brian-j-smith.github.io/drat
LazyData: true
License: GPL-3
URL: https://brian-j-smith.github.io/MachineShop/
BugReports: https://github.com/brian-j-smith/MachineShop/issues
VignetteBuilder: knitr
Encoding: UTF-8
Collate: 'classes.R' 'conditions.R' 'MachineShop-package.R'
        'MLControl.R' 'MLInput.R' 'MLMetric.R' 'MLModel.R'
        'MLOptimization.R' 'ML_AdaBagModel.R' 'ML_AdaBoostModel.R'
        'ML_BARTMachineModel.R' 'ML_BARTModel.R' 'ML_BlackBoostModel.R'
        'ML_C50Model.R' 'ML_CForestModel.R' 'ML_CoxModel.R'
        'ML_EarthModel.R' 'ML_FDAModel.R' 'ML_GAMBoostModel.R'
        'ML_GBMModel.R' 'ML_GLMBoostModel.R' 'ML_GLMModel.R'
        'ML_GLMNetModel.R' 'ML_KNNModel.R' 'ML_LARSModel.R'
        'ML_LDAModel.R' 'ML_LMModel.R' 'ML_MDAModel.R' 'ML_NNetModel.R'
        'ML_NaiveBayesModel.R' 'ML_ParsnipModel.R' 'ML_PLSModel.R'
        'ML_POLRModel.R' 'ML_QDAModel.R' 'ML_RFSRCModel.R'
        'ML_RPartModel.R' 'ML_RandomForestModel.R' 'ML_RangerModel.R'
        'ML_SVMModel.R' 'ML_StackedModel.R' 'ML_SuperModel.R'
        'ML_SurvRegModel.R' 'ML_TreeModel.R' 'ML_XGBModel.R'
        'ModelFrame.R' 'ModelRecipe.R' 'ModelSpecification.R'
        'TrainedInputs.R' 'TrainedModels.R' 'TrainingParams.R'
        'append.R' 'calibration.R' 'case_comps.R' 'coerce.R'
        'combine.R' 'confusion.R' 'convert.R' 'data.R' 'dependence.R'
        'diff.R' 'expand.R' 'extract.R' 'fit.R' 'grid.R' 'metricinfo.R'
        'metrics.R' 'metrics_factor.R' 'metrics_numeric.R'
        'modelinfo.R' 'models.R' 'performance.R' 'performance_curve.R'
        'plot.R' 'predict.R' 'print.R' 'recipe_roles.R' 'reexports.R'
        'resample.R' 'response.R' 'rfe.R' 'settings.R' 'step_kmeans.R'
        'step_kmedoids.R' 'step_lincomp.R' 'step_sbf.R' 'step_spca.R'
        'summary.R' 'survival.R' 'utils.R' 'varimp.R'
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-09-04 13:20:49 UTC; bjsmith
Repository: CRAN
Date/Publication: 2026-09-04 15:50:19 UTC
